# -*- coding: utf-8 -*-
import json
import copy
import os.path
import logging

from collections import namedtuple
from ecimporter.lib.validators import get_external_validators
from efa.model.project import Project
from ringo.lib.helpers import get_app_location
from formbar.config import Config, load
from formbar.form import (
    Form,
    Dependency,
    Registry
)
from ecimporter.lib.config import Participant, Outputindicator, Ignored


log =logging.getLogger(__name__)

Item = namedtuple("Item", "id project participant")
Participant_ = namedtuple("Participant", "participations")

def create_dependencies(request, item):
    return Dependency(request.params,
                      request.db,
                      request.registry.settings,
                      request.translate,
                      item,
                      request.method,
                      request.accept_language,
                      Registry(request.registry.settings))



def report_differences(missing_fields, new_fields):
    """
    builds response for differing fields

    :param missing_fields:
    :param new_fields:
    :return: msg dict
    """
    title = "Found new keys or missing (%s, %s)" % (
        repr(new_fields), repr(missing_fields)
    )
    payload = {"new_keys": repr(new_fields),
               "missing_keys": repr(missing_fields)}
    return dict(status="fail", title=title, payload=payload)


def check_input_differences(json_str):
    fields_sent = set(json.loads(json_str)[0])
    complete_fields = set(Outputindicator.keys()) | \
        set(Participant.keys()) | \
        set(Ignored.keys()) | \
        set(["vertragsabschluss"])
    new_fields = fields_sent - complete_fields
    missing_fields = complete_fields - fields_sent
    return (new_fields or missing_fields,
            report_differences(missing_fields, new_fields))


def log_validation_errors(errors, form_type=None, personal_id=None):
    preamble = "formtype: {}, personal_id: {}".format(form_type, personal_id)
    log.warn(preamble)
    for error, msg in errors.items():
        log.warn("{} {}".format(error, msg))


def log_validation_warnings(warnings, form_type=None, personal_id=None):
    preamble = "formtype: {}, personal_id: {}".format(form_type, personal_id)
    log.warn(preamble)
    for warn, msg in warnings.items():
        log.warn("{} {}".format(warn, msg))


def check_participant_form(data, request):
    data = copy.deepcopy(data)
    config = Config(load(os.path.join(get_app_location("efa"),
                                      "efa", "views",
                                      "forms", "participants.xml")))
    config_form = config.get_form("update")
    dependencies = create_dependencies(request, None)
    form = Form(config_form, request=request, dependencies=dependencies)
    res = form.validate(data)
    if not res:
        log_validation_errors(form.get_errors(), "participant", data['personal_id']),
    return res


def modify_warning(warnings, ignored=None):
    if ignored is None:
        ignored = []
    filtered_warnings = {}
    for key in warnings:
        if not key in ignored:
            filtered_warnings[key] = warnings[key]
    return filtered_warnings


def retrieve_project(request):
    project = request.db.query(Project).one()
    return project

def check_participation_form(pt, pn, oi, request):
    # copy data, because formbar changes the dicts
    pt = copy.deepcopy(pt)
    pn = copy.deepcopy(pn)
    oi = copy.deepcopy(oi)

    # build participation form
    config = Config(load(os.path.join(get_app_location("ec"),
                                      "ec", "views", "forms",
                                      "participations.xml")))
    config_form = config.get_form("update")

    # merge the dicts, to have one data dict.
    for key in pt:
        pn["participant." + key] = pt[key]
    for key in oi:
        pn["ecoi." + key] = oi[key]

    project = retrieve_project(request)
    # DO NOT EVEN THINK OF ASKING WHY
    #
    # I sold my soul with this code!
    #
    # It is assumed safe no participation/participant has id=0
    # project is the current running project
    # Participant is a fake participant object
    #
    item = Item(0, project, Participant_([]))
    dependencies = create_dependencies(request, item)
    form = Form(config_form, values=pn, request=request, dependencies=dependencies)
    form.validate()
    ignored = ["end_date", "planned_end_date", "ecoi.course"]
    filtered_warnings  = modify_warning(form.get_warnings(), ignored=ignored)
    has_problems = form.has_errors() or filtered_warnings
    personal_id = pt['personal_id']
    if form.has_errors():
        log_validation_errors(form.get_errors(), "participation", personal_id)
    if filtered_warnings:
        log_validation_warnings(filtered_warnings, "participation",
                                personal_id)
    return not has_problems
